package examplemodifie;

import java.awt.*;  
import java.awt.image.BufferedImage;  
import javax.swing.*;  
import javax.swing.tree.*;  
   
public class ImageTest extends JPanel  
{  
    BufferedImage image;  
    JTree tree;  
    Dimension size;  
   
    public ImageTest()  
    {  
        createTree();  
        size = new Dimension(200, 200);  
    }  
   
    protected void paintComponent(Graphics g)  
    {  
        super.paintComponent(g);  
        if(image == null)  
            initImage();  
        g.drawImage(image, 0, 0, this);  
    }  
   
    private void initImage()  
    {  
        int w = size.width, h = size.height;  
        // another possibility  
        //Dimension d = tree.getPreferredSize();  
        //int w = d.width, h = d.height;  
        System.out.println("image width = " + w + "\timage height = " + h);  
        image = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);  
        Graphics2D g2 = image.createGraphics();  
        g2.setPaint(Color.pink);  
        g2.fillRect(0, 0, w, h);  
        // locate tree inside image  
        //g2.translate(x, y);  
        tree.paint(g2);  
        g2.dispose();  
    }  
   
    private void createTree()  
    {  
        String[] birds = {  
            // branches |<-- child leaf nodes -->|  
            "hawks", "gray", "red-tailed", "rough-legged",  
            "falcons", "harrier", "kestrel", "kite",   
            "owls", "barred", "saw-whet", "snowy"  
        };  
        DefaultMutableTreeNode root = new DefaultMutableTreeNode("birds");  
        DefaultMutableTreeNode[] nodes = new DefaultMutableTreeNode[birds.length];  
        for(int j = 0; j < nodes.length; j++)  
            nodes[j] = new DefaultMutableTreeNode(birds[j]);  
        for(int j = 0; j < 9; j += 4)  
        {  
            root.insert(nodes[j], j % 3);  
            for(int k = j + 1; k < j + 4; k++)  
                nodes[j].insert(nodes[k], k - j - 1);  
        }  
        DefaultTreeModel model = new DefaultTreeModel(root);  
        tree = new JTree(model);  
    }  
   
    public Dimension getPreferredSize()  
    {  
        return size;  
    }  
   
    public static void main(String[] args)  
    {  
        ImageTest it = new ImageTest();  
        JFrame f = new JFrame();  
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);  
        f.setLayout(new GridBagLayout());  
        GridBagConstraints gbc = new GridBagConstraints();  
        gbc.weightx = 1.0;  
        gbc.gridwidth = GridBagConstraints.RELATIVE;  
        f.add(it.tree, gbc);  
        gbc.gridwidth = GridBagConstraints.REMAINDER;  
        f.add(it, gbc);  
        f.setSize(500,400);  
        f.setLocation(200,200);  
        f.setVisible(true);  
        System.out.println("cp width    = "  + f.getContentPane().getWidth() + "\t" +  
                           "cp height    = " + f.getContentPane().getHeight() + "\n" +  
                           "tree width  = "  + it.tree.getWidth() + "\t" +   
                           "tree height  = " + it.tree.getHeight());  
    }  
} 
