package software;

import javax.swing.text.AttributeSet;
import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;

import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import java.awt.Font;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultCaret;
import javax.swing.text.Document;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

import software.util.MinSizeBorderLayoutPanel;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

@SuppressWarnings("serial")
public class JPanelConsole extends JPanel {
	
	private static JTextPane textPane;
		
	public final static SimpleAttributeSet WARNING = new SimpleAttributeSet();
	public final static SimpleAttributeSet GOOD = new SimpleAttributeSet();
	

	/**
	 * Create the console panel.
	 */
	public JPanelConsole() {
		super();
		build();
	}
	private void build() {
		//init styles
	    StyleConstants.setForeground(WARNING, Color.red);
	    StyleConstants.setBold(WARNING, true);
	    StyleConstants.setForeground(GOOD, Color.blue);
	    //StyleConstants.setBold(blue, true);
		
		setLayout(new BorderLayout(0, 0));
		JScrollPane scrollPane = new JScrollPane();
		add(scrollPane, BorderLayout.CENTER);
		textPane = new JTextPane();
		textPane.setMinimumSize(new Dimension(300, 50));
		DefaultCaret caret = (DefaultCaret)textPane.getCaret();//pour défillement automatique du scroll
		caret.setUpdatePolicy(DefaultCaret.ALWAYS_UPDATE);//pour défillement automatique du scroll
		scrollPane.setViewportView(textPane);
		textPane.setAlignmentY(Component.TOP_ALIGNMENT);
		textPane.setEditable(false);
		
		JButton btnTestConsole = new JButton("TestConsole");
		btnTestConsole.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				printConsoleLogLine("1", null);
				System.out.println("1");
			}
		});

		add(btnTestConsole, BorderLayout.EAST);
		btnTestConsole.setFont(new Font("Dialog", Font.BOLD, 10));
		
		JPanel panelNorth = new JPanel();
		add(panelNorth, BorderLayout.NORTH);
		panelNorth.setLayout(new BorderLayout(0, 0));
		JLabel lblCc = new JLabel(" Console ");
		lblCc.setFont(new Font("Dialog", Font.ITALIC, 8));
		panelNorth.add(lblCc, BorderLayout.WEST);
		
		JButton btnClear = new JButton("clear");
		btnClear.setFont(new Font("Dialog", Font.BOLD | Font.ITALIC, 8));
		btnClear.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				clearConsoleLog();
				System.out.println("clear Console Document Content");
			}
		});
		panelNorth.add(btnClear, BorderLayout.EAST);
		
		setMinimumSize(new MinSizeBorderLayoutPanel(this));
				
	}
	
	/**
	 * Efface le contenu de la console.
	 */
	  protected static void clearConsoleLog() {
		    Document d = textPane.getDocument();
		    try {
		      d.remove(0,d.getLength());
		    } catch (BadLocationException ble) {
		    }
		  }
	

	  /**Ecrit dans le textPane (pas de retour a la ligne!!).
	 * @param s
	 * @param attributes
	 */
	protected static void printConsoleLog(String s, AttributeSet attributes) {
		    Document d = textPane.getDocument();
		    try {
		      d.insertString(d.getLength(), s, attributes);
		    } catch (BadLocationException ble) {
		    }
		  }


	  /** Ecrit dans le textPane une ligne (texte + \r\n ).
	 * @param s
	 * @param attributes
	 * <code> // create some handy attribute sets
    SimpleAttributeSet red = new SimpleAttributeSet();
    StyleConstants.setForeground(red, Color.red);
    StyleConstants.setBold(red, true);
    // add the text
    append("NULL ", null);
    append("red", red); </code>
	 */
	public void printConsoleLogLine(String s, AttributeSet attributes) {
		  printConsoleLog(s + "\r\n", attributes);
		  }

}
