package software.util;

import java.awt.Dimension;
import java.util.Arrays;
import java.util.Comparator;

public class Math {

	/** Sort a two columns array of by second column and return the sorted array.<p>
	 *  Example:<p>
	 *  <code>Integer[][] arrayRedX = {{0,20} ,{1,6} , {2,0} , {3,55} , {4,-6}};
		arrayRedX = intArraySortAscending(arrayRedX);</code><p>
	 *  0, 20	|	4, -6<br>
	 *  1,  6		|	2,  0<br>
	 *  2,  0		>	1,  6<br>
	 *  <b>3</b> ,55	|	0, 20<br>
	 *  4, -6	|	<b>3</b>, 55<br><p>
	 *  The result for the reference (first column) max value (second column) is : <code>arrayRedX[arrayRedX.length -1][0]</code>. Here this reference is <b>3</b>.
	 * @param arrayToSort	The array to sort. Type must be an <i>Integer[][]</i>.
	 * @return The array sorted. Type is <i>Integer[][]</i>.
	 */
	public static Integer[][] intArraySortAscending(Integer[][] arrayToSort) {
//			for (Integer[] i : arrayToSort) {
//	        System.out.println(i[0] + "," + i[1]);
//	    	}
			
	        Arrays.sort(arrayToSort, new Comparator<Integer[]>() {
	            @Override
	            public int compare(Integer[] int1, Integer[] int2) {
	                Integer numOfKeys1 = int1[1];
	                Integer numOfKeys2 = int2[1];
	                return numOfKeys1.compareTo(numOfKeys2);
	            }
	        });
	        
//			System.out.println("================");
//			for (Integer[] i : arrayToSort) {
//	            System.out.println(i[0] + "," + i[1]);
//	        }
			
	        return arrayToSort;
	    }
	
	/** double Parabolic interpolation with 3 points (at equal dist).<p>
	 * Simplication of {@link FullSubpixelAccuracy}.<p>
	 * <b>Careful</b> : if <i>y1 + y3 - 2*y2 = 0</i>  (case of 3 points in a alignment) then the return
	 * become a non interpolation ( xmax = 0 and Ymax = y2 )
	 * @param y1
	 * @param y2 actual max
	 * @param y3
	 * @return float[2] = { <li>xmax <i>correction for actual abscissa</i> ,
	 * 						<li> ymax <i>value of the virtual max</i> }
	 */
	public static double[] SubpixelParabolicInterpolation(int y1, int y2, int y3){
		double result[] = new double[2];
		double xv, yv;
		double diff  = y1 - y3;
		double denum   = y1 + y3 - 2*y2;
		if (denum == 0){
			result[0]=0; result[1]=y2;
		}else{
			xv = (diff) / (2*denum);
			yv = y2 - ( ((diff)*(diff)) / ( 8*denum ) );
			result[0]=xv; result[1]=yv;
		}
		return result;
		
	}
	
	/** float Parabolic interpolation with 3 points (at equal dist).<p>
	 * Simplication of {@link FullSubpixelAccuracy}.<p>
	 * <b>Careful</b> : if <i>y1 + y3 - 2*y2 = 0</i>  (case of 3 points in a alignment) then the return
	 * become a non interpolation ( xmax/min = 0 and Ymax/min = y2 )
	 * @param y1
	 * @param y2 actual max
	 * @param y3
	 * @return float[2] = { <li>xmax/min <i>correction for actual abscissa</i> ,
	 * 						<li> ymax/min <i>value of the virtual max</i> }
	 */
	public static float[] FloatSubpixelParabolicInterpolation(int y1, int y2, int y3){
		float result[] = new float[2];
		float xv, yv;
		float diff  = y1 - y3;
		float denum   = y1 + y3 - 2*y2;
		if (denum == 0){
			result[0]=0; result[1]=y2;
		}else{
			xv = (diff) / (2*denum);
			yv = y2 - ( ((diff)*(diff)) / ( 8*denum ) );
			result[0]=xv; result[1]=yv;
		}
		return result;
	}
	
	/**
	 * The general form of a parabola is given by the equation: A * x^2 + B * x
	 * + C = y where A, B, and C are arbitrary Real constants.<p>
	 *  You have three pairs of points that are (x,y) ordered pairs. Substitute the x and y
	 * values of each point into the equation for a parabola. You will get three
	 * LINEAR equations in three unknowns, the three constants. You can then
	 * easily solve this system of three equations for the values of A, B, and
	 * C, and you'll have the equation of the parabola that intersects your 3
	 * points. The vertex is where the first derivative is 0, a little algebra
	 * gives: <li> ( -B/2A , C - B^2/4A ) for the vertex.
	 * <b>Careful</b> : if <i>(x3 * (y2 - y1) + x2 * (y1 - y3) + x1 * (y3 - y2)) / (x1 - x2) * (x1 - x3) * (x2 - x3) = 0</i>  (case of 3 points in a alignment) then the return
	 * become a non interpolation ( xmax/min = x2 and Ymax/min = y2 )
	 * 
	 * @param x1
	 * @param y1
	 * @param x2
	 * @param y2
	 * @param x3
	 * @param y3
	 * @return double[2] = { <li>xmax/min <i>abscissa of the maximum/minimum</i> ,
	 * 						<li> ymax/min <i>value of the calculate max/min</i> }
	 */
	public static double[] ParabolicInterpolation(int x1, int y1, int x2, int y2, int x3, int y3){
	
	double result[] = new double[2];
	double xv, yv;
	double denom, A, B, C;
	denom = (x1 - x2) * (x1 - x3) * (x2 - x3);
	A     = (x3 * (y2 - y1) + x2 * (y1 - y3) + x1 * (y3 - y2)) / denom;
	//System.out.println("A "+A);
	if (A == 0) {
		xv = x2;
		yv = y2;
	}else{
		B     = (x3*x3 * (y1 - y2) + x2*x2 * (y3 - y1) + x1*x1 * (y2 - y3)) / denom;
		//System.out.println("B "+B);
		C     = (x2 * x3 * (x2 - x3) * y1 + x3 * x1 * (x3 - x1) * y2 + x1 * x2 * (x1 - x2) * y3) / denom;
		//System.out.println("C "+C);
		xv = -B / (2*A);
		yv = C - B*B / (4*A);
		
	}
	result[0]=xv; result[1]=yv;

	return result;
	
}
	
	public static void main(String[] args) {
		Integer[][] arrayRedX = {{0,20} ,{1,6} , {2,0} , {3,55} , {4,-6}};
		arrayRedX = intArraySortAscending(arrayRedX);
		double[] r = ParabolicInterpolation(-1, 5,    0, 4,   1, 6 );
		System.out.println("x"+ r[0] + " y "+ r[1]);
		double[] r1 = SubpixelParabolicInterpolation( 5,    4,    6);
		System.out.println("x"+ r1[0]+" y "+r1[1] );
		float[] r2 = FloatSubpixelParabolicInterpolation( 5,    4,    6);
		System.out.println("x"+ r2[0]+" y "+r2[1] );
	}


}
