package software.util;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.util.Arrays;

import javax.swing.JPanel;

@SuppressWarnings("serial")
public class MinSizeBorderLayoutPanel extends Dimension {
	

	
	/** Give minimumDimension for a JPanel with BorderLayout<p> 
	 * <ul>Calculate and return {@link Dimension} corresponding to all children's components of a {@link JPanel} with a {@link BorderLayout}
	 * , according with their minimums dimensions ( <code>getMinimumSize</code> ). 
	 *  </ul>
	 * @param jpanel (whith {@link BorderLayout})
	 */
	public MinSizeBorderLayoutPanel(JPanel panel) {
		super();
		
		assert panel.getLayout().getClass()== BorderLayout.class;
		
		int height = 0;
		int width = 0;
		
		int height_WCE[] = { 0, 0, 0 };
		int width_NwceS[] = { 0, 0, 0 };
		
		//System.out.println("found component in " + panel.getClass().getName() + " >>>>" );
		
		String otherPlaces[] = { "Center", "North", "South", "West", "East" };
        for (String s : otherPlaces)
        {
          Component c2 = getComponent(panel, s);
          if (c2 != null) {
        	  //System.out.println("found component at " + s + ", " + c2.toString() );
        	  switch (s) {
			case "West":
				height_WCE[0] = c2.getMinimumSize().height;
				width_NwceS[1] += c2.getMinimumSize().width;
				break;
			case "Center":
				height_WCE[1] = c2.getMinimumSize().height;
				width_NwceS[1] += c2.getMinimumSize().width;
				break;
			case "East":
				height_WCE[2] = c2.getMinimumSize().height;
				width_NwceS[1] += c2.getMinimumSize().width;
				break;
			case "North":
				height += c2.getMinimumSize().height;
				width_NwceS[0] = c2.getMinimumSize().width;
				break;
			case "South":
				height += c2.getMinimumSize().height;
				width_NwceS[2] = c2.getMinimumSize().width;
				break;
			default:
				break;
			}
          }
        }
//        System.out.println("height_WCE  : "+ Arrays.toString(height_WCE));
//        System.out.println("width_NwceS : "+ Arrays.toString(width_NwceS));
        
        Arrays.sort(height_WCE);
        Arrays.sort(width_NwceS);
        
//        System.out.println("height_WCE  <: "+ Arrays.toString(height_WCE));
//        System.out.println("width_NwceS <: "+ Arrays.toString(width_NwceS));
        
        height	 = height_WCE[2] + height;
        width	 = width_NwceS[2] + width;
        
		setSize(width, height);
	}

	private Component getComponent(JPanel panel, String constraint)
    {
      BorderLayout bl = (BorderLayout)panel.getLayout();
      return bl.getLayoutComponent(constraint);
    }
	
}
